SystemOrganization addCategory: #'WinReg-Core'!

Object subclass: #WinReg
	instanceVariableNames: 'key subKey'
	classVariableNames: 'HKeyClassesRoot HKeyCurrentUser HKeyLocalMachine HKeyUsers KeyAllAccess KeyCreateLink KeyCreateSubKey KeyEnumerateSubKeys KeyExecute KeyNotify KeyQueryValue KeyRead KeySetValue KeyWrite ReadControl RegBinary RegDword RegDwordBigEndian RegDwordLittleEndian RegExpandSz RegFullResourceDescriptor RegLink RegMultiSz RegNone RegOptionBackupRestore RegOptionNonVolatile RegOptionVolatile RegQword RegQwordLittleEndian RegResourceList RegResourceRequirementsList RegSz StandardRightsAll StandardRightsRead StandardRightsWrite Synchronize'
	poolDictionaries: ''
	category: 'WinReg-Core'!

!WinReg commentStamp: 'Richo 6/8/2011 17:14' prior: 0!
I represent the windows registry.

Code examples:

"Get windows current version"
(WinReg at: 'HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion') values

"Enumerate the available serial ports"
(WinReg hKeyLocalMachine at: 'HARDWARE\DEVICEMAP\SERIALCOMM') values!

!WinReg class methodsFor: 'api access' stamp: 'Richo 6/8/2011 09:58'!
apiRegCloseKey: hKey
	<apicall: long 'RegCloseKey' (long)  module: 'advapi32.dll'>
	self externalCallFailed! !

!WinReg class methodsFor: 'api access' stamp: 'Richo 6/8/2011 09:58'!
apiRegCreateKeyExA: hKey with: lpSubKey with: reserved with: lpClass with: dwOptions with: samDesired with: lpSecurityAttributes with: phkResult with: lpdwDisposition
	<apicall: long 'RegCreateKeyExA' (long char* long char* long long long long long) module: 'advapi32.dll'>
	self externalCallFailed! !

!WinReg class methodsFor: 'api access' stamp: 'Richo 6/8/2011 09:58'!
apiRegDeleteKeyA: hKey with: lpSubKey
	<apicall: long 'RegDeleteKeyA' (long char*)  module: 'advapi32.dll'>
	self externalCallFailed! !

!WinReg class methodsFor: 'api access' stamp: 'Richo 6/8/2011 13:02'!
apiRegEnumKeyExA: hkey with: dwIndex with: lpName with: lpcName with: lpReserved with: lpClass with: lpcClass with: lpftLastWriteTime
<apicall: long 'RegEnumKeyExA' (long long char* long long char* long long)  module: 'advapi32.dll'>
	self externalCallFailed! !

!WinReg class methodsFor: 'api access' stamp: 'Richo 6/8/2011 13:02'!
apiRegEnumValue: hKey with: dwIndex with: lpValueName with: lpcchValueName with: lpReserved with: lpType with: lpData with: lpcbData
<apicall: long 'RegEnumValueA' (long long long long long long long long)  module: 'advapi32.dll'>
	self externalCallFailed! !

!WinReg class methodsFor: 'api access' stamp: 'Richo 6/8/2011 09:58'!
apiRegOpenKeyExA: hKey with: lpSubKey with: ulOptions with: samDesired with: phkResult
	<apicall: long 'RegOpenKeyExA' (long char* long long long)  module: 'advapi32.dll'>
	self externalCallFailed! !

!WinReg class methodsFor: 'instance creation' stamp: 'Richo 6/8/2011 13:02'!
at: fullPath
| key subKey |
key := fullPath upTo: $\.
subKey := fullPath allButFirst: key size + 1.
^ (key caseOf: {
	['HKEY_CLASSES_ROOT'] -> [self hKeyClassesRoot].
	['HKEY_CURRENT_USER'] -> [self hKeyCurrentUser].
	['HKEY_LOCAL_MACHINE'] -> [self hKeyLocalMachine].
	['HKEY_USERS'] -> [self hKeyUsers]
	}) at: subKey! !

!WinReg class methodsFor: 'instance creation' stamp: 'Richo 6/8/2011 12:33'!
hKeyClassesRoot
	^ (self basicNew setKey: HKeyClassesRoot) initialize! !

!WinReg class methodsFor: 'instance creation' stamp: 'Richo 6/8/2011 12:33'!
hKeyCurrentUser
	^ (self basicNew setKey: HKeyCurrentUser) initialize! !

!WinReg class methodsFor: 'instance creation' stamp: 'Richo 6/8/2011 12:33'!
hKeyLocalMachine
	^ (self basicNew setKey: HKeyLocalMachine) initialize! !

!WinReg class methodsFor: 'instance creation' stamp: 'Richo 6/8/2011 12:34'!
hKeyUsers
	^ (self basicNew setKey: HKeyUsers) initialize! !

!WinReg class methodsFor: 'class initialization' stamp: 'Richo 6/8/2011 16:13'!
initialize
	"
	self initialize.
	"
	HKeyClassesRoot := -2147483648.
	HKeyCurrentUser := -2147483647.
	HKeyLocalMachine := -2147483646.
	HKeyUsers := -2147483645.
	KeyAllAccess := 983103.
	KeyCreateLink := 32.
	KeyCreateSubKey := 4.
	KeyEnumerateSubKeys := 8.
	KeyExecute := 131097.
	KeyNotify := 16.
	KeyQueryValue := 1.
	KeyRead := 131097.
	KeySetValue := 2.
	KeyWrite := 131078.
	ReadControl := 131072.
	RegOptionBackupRestore := 4.
	RegOptionNonVolatile := 0.
	RegOptionVolatile := 1.
	StandardRightsAll := 2031616.
	StandardRightsRead := 131072.
	StandardRightsWrite := 131072.
	Synchronize := 1048576.
	RegNone :=  0.
	RegSz :=  1.
	RegExpandSz :=  2 .
	RegBinary :=  3 .
	RegDword :=  4.
	RegDwordLittleEndian :=  4.
	RegDwordBigEndian :=  5 .
	RegLink :=  6.
	RegMultiSz :=  7.
	RegResourceList :=  8.
	RegFullResourceDescriptor :=  9.
	RegResourceRequirementsList :=  10.
	RegQword :=  11.
	RegQwordLittleEndian :=  11.! !

!WinReg class methodsFor: 'testing' stamp: 'Richo 6/8/2011 12:29'!
test1
	| result |
	result := ExternalAddress allocate: 4.
	self apiRegOpenKeyExA: HKeyCurrentUser
		 with: 'Software\Richo'
		 with: 0
		 with: KeyAllAccess
		 with: result asInteger.
	(result signedLongAt: 1) = 0
		ifTrue: [
		self apiRegCreateKeyExA: HKeyCurrentUser
			 with: 'Software\Richo'
			 with: 0
			 with: 'REG_DWORD'
			 with: RegOptionNonVolatile
			 with: KeyAllAccess
			 with: 0
			 with: result asInteger
			 with: 0.
		(result signedLongAt: 1) = 0
			ifTrue: [self inform: 'Error while creating the key'. ^ self].
		self inform: 'Key created...'
		].
	self inform: 'Deleting key...'.
	self apiRegDeleteKeyA: (result signedLongAt: 1) with: ''.
	self apiRegCloseKey: (result signedLongAt: 1)! !

!WinReg methodsFor: 'accessing' stamp: 'Richo 6/8/2011 12:35'!
at: aString
	subKey := aString! !

!WinReg methodsFor: 'private' stamp: 'Richo 6/8/2011 17:16'!
format: aByteArray type: type
	^ type caseOf: {
		[RegSz] -> [self formatString: aByteArray].
		[RegExpandSz] -> [self formatString: aByteArray].
		[RegDword] -> [aByteArray unsignedLongAt: 1].
		"Other types should be implemented"
	} otherwise: [aByteArray] ! !

!WinReg methodsFor: 'private' stamp: 'Richo 6/9/2011 13:16'!
formatString: aByteArray
	^ (aByteArray reject: [:each | each = 0]) asString! !

!WinReg methodsFor: 'initialize-release' stamp: 'Richo 6/8/2011 12:35'!
initialize
	super initialize.
	subKey := ''! !

!WinReg methodsFor: 'private' stamp: 'Richo 6/8/2011 12:34'!
setKey: aNumber
	key := aNumber! !

!WinReg methodsFor: 'accessing' stamp: 'Richo 6/15/2011 17:23'!
values
| pairs result cont valueName valueData nameSize dataSize valueType |
pairs := OrderedCollection new.
result := ExternalAddress gcallocate: 4.
cont := 0.
(self class apiRegOpenKeyExA: key
	 with: subKey
	 with: 0
	 with: KeyAllAccess
	 with: result asInteger) = 0
		ifTrue: [[
		valueName := ExternalAddress gcallocate: 260.
		valueData := ExternalAddress gcallocate: 260.
		valueType := ExternalAddress gcallocate: 4.
		nameSize := ExternalAddress gcallocate: 4.
		dataSize := ExternalAddress gcallocate: 4.
		(self class apiRegEnumValue: (result signedLongAt: 1) 
			with: cont
			with: valueName asInteger 
			with: nameSize asInteger 
			with: 0
			with: valueType asInteger
			with: valueData asInteger
			with: dataSize asInteger) ~= 259 "ERROR_NO_MORE_ITEMS"]
				whileTrue: [cont := cont + 1.
				valueName := String streamContents: [:stream |
					1 to: (nameSize unsignedLongAt: 1) do: [:i |
						stream nextPut: (valueName unsignedCharAt: i)]].
				valueData := ByteArray streamContents: [:stream |
					1 to: (dataSize unsignedLongAt: 1) do: [:i |
						stream nextPut: (valueData byteAt: i)]].
				pairs add: valueName -> (self format: valueData type: (valueType unsignedLongAt: 1)).
				]].
self class apiRegCloseKey: (result signedLongAt: 1).	
^ pairs as: Dictionary! !
WinReg initialize!
